#include "i2c.h"
#include "tmr.h"
#include "utils.h"

extern CI2C gI2C;

#define SENSOR_FOSP01_SAMPLE_DELAY_MS   50

void CSensor::Init()
   {
   m_nSmpCnt = 0;
   m_State = SENSOR_ST_IDLE;
   m_nKLPFA = 4;
   m_nKLPFC = SENSOR_DFL_KLPFC;
   m_nPZTblSel = SENSOR_DFL_PZTBL_SEL;
   m_nAltM = m_nAltCm = m_nAltCmx16 = 0;
   m_nCmPerSec = m_nCmPerSecx16 = 0;
   m_nWindowSecs = SENSOR_DFL_WINDOW_SECS;
   m_nWindowSamples = m_nWindowSecs * SENSOR_SAMPLES_PER_SEC;
   m_nTempCorrInx = SENSOR_DFL_TEMP_CORR_INX;

   // Set SENSOR_XCLR as write 

   PORT_REG |= (SENSOR_XCLR_WR);
   SET_XCLR_LO();
   ReadCoeffs();
   }


void CSensor::ReadCoeffs()
   {
   u8 buf[2];
   
   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,16,2,buf);
   m_C1 = int(buf[1]) | (int(buf[0])<<8);

   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,18,2,buf);
   m_C2 = int(buf[1]) | (int(buf[0])<<8);

   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,20,2,buf);
   m_C3 = int(buf[1]) | (int(buf[0])<<8);

   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,22,2,buf);
   m_C4 = int(buf[1]) | (int(buf[0])<<8);

   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,24,2,buf);
   m_C5 = int(buf[1]) | (int(buf[0])<<8);

   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,26,2,buf);
   m_C6 = int(buf[1]) | (int(buf[0])<<8);

   gI2C.RcvBuf(I2C_ID_FOSP01_COEFF,28,2,buf);
   m_C7 = int(buf[1]) | (int(buf[0])<<8);

   m_A = (int)gI2C.RcvByte(I2C_ID_FOSP01_COEFF,30);
   m_B = (int)gI2C.RcvByte(I2C_ID_FOSP01_COEFF,31);
   m_C = (int)gI2C.RcvByte(I2C_ID_FOSP01_COEFF,32);
   m_D = (int)gI2C.RcvByte(I2C_ID_FOSP01_COEFF,33);
   }

void CSensor::Reset()
   {
   PORT_REG &= ~SENSOR_XCLR;
   I2C_DELAY();
   PORT_REG |= SENSOR_XCLR;
   }

void CSensor::TriggerPressureSample()
   {
   SET_XCLR_HI();
   BusyWait(10);
   gI2C.XmtByte(I2C_ID_FOSP01_ADC,0xFF,0xF0);
   }

void CSensor::TriggerTemperatureSample()
   {
   SET_XCLR_HI();
   BusyWait(10);
   gI2C.XmtByte(I2C_ID_FOSP01_ADC,0xFF,0xE8);
   }

int CSensor::ReadPressure()
   {
   u8 buf[2];
   int nPres;
   gI2C.RcvBuf(I2C_ID_FOSP01_ADC,0xFD,2,buf);
   SET_XCLR_LO();
   BusyWait(10);
   nPres = int(buf[1]) | (int(buf[0])<<8);
   return nPres; 
   }


int CSensor::ReadTemperature()
   {
   u8 buf[2];
   int nTemp;
   gI2C.RcvBuf(I2C_ID_FOSP01_ADC,0xFD,2,buf);
   SET_XCLR_LO();
   BusyWait(10);
   nTemp = int(buf[1]) | (int(buf[0])<<8);
   return nTemp; 
   }


void CSensor::ConvertPressureTemperature(int pzInx, int tcInx)
   {
   int dUT,tmp,OFF,SENS,X,P,T;
   tmp = (m_D2 - m_C5);
   if (tmp >= 0)
      {
      dUT = tmp - ((((tmp*tmp)>>14)*m_A) >> m_C);
      }
   else
      {
      dUT = tmp - ((((tmp*tmp)>>14)*m_B) >> m_C);
      }
   OFF = (m_C2 + (((m_C4-1024)*dUT)>>14)) << 2;
   SENS = m_C1 + ((m_C3*dUT)>>10);
   X = ((SENS*(m_D1-7168))>>14) - OFF;
   P = ((X*100)>>5) + (m_C7*10);
   T = 250 + ((dUT*m_C6)>>16) - (dUT>>m_D);

   m_SmpBuf[m_nSmpCnt].tck = (int)gTmr.ticks;
   m_SmpBuf[m_nSmpCnt].p = m_nPresPaSample = P;
   m_SmpBuf[m_nSmpCnt].t = m_nTempCx10Sample = T;
#ifdef USE_Z_REGRESSION
   m_SmpBuf[m_nSmpCnt].z = Pa2Cm(m_nPresPaSample, pzInx, tcInx);
#endif

   // increment the pointer, now it points to the oldest sample in the circular buffer
   m_nSmpCnt++;
   if (m_nSmpCnt == m_nWindowSamples)
      {
//       ham_DrawText(0,0,"t%06d",((gTmr.ticks-sTick)*TMR_TICK_MS)/m_nWindowSamples);
      m_nSmpCnt = 0;
//       sTick = gTmr.ticks;
      }
   }


void CSensor::InitData()
   {
   InitWindowBuffers();
   m_State = SENSOR_ST_IDLE;
   }
   
void CSensor::InitWindowBuffers()
   {
   int n,tck;
   AcquireAveragedSample(4, m_nPZTblSel, m_nTempCorrInx);
   tck = int(gTmr.ticks);
   m_nSmpCnt =  m_nWindowSamples-1;
   n = m_nWindowSamples;
   while(n--)
      {
      m_SmpBuf[n].z  = m_nAltCm;
      m_SmpBuf[n].p  = m_nPresPa;
      m_SmpBuf[n].t = m_nTempCx10;
      m_SmpBuf[n].tck = tck - m_nWindowSamples + n;
      }
   }
   

void CSensor::AcquireAveragedSample(int nSamples, int pzInx, int tcInx)
   {
   int pAccum, tAccum, n;

   n = nSamples;
   pAccum = 0;
   tAccum = 0;
   while(n--)
      {
      TriggerPressureSample();
      // 35mS for conversion, datasheet specs 40mS wait
      BusyWait(SENSOR_FOSP01_SAMPLE_DELAY_MS);
      m_D1   = ReadPressure();
      TriggerTemperatureSample();
      BusyWait(SENSOR_FOSP01_SAMPLE_DELAY_MS);
      m_D2 = ReadTemperature();
      ConvertPressureTemperature(pzInx, tcInx);
      pAccum += m_nPresPaSample;
      tAccum += m_nTempCx10Sample;
      ham_DrawText(0,n,"%d %d",m_D1,m_D2);
      }
   m_nPresPa = pAccum/nSamples;
   m_nTempCx10 = tAccum/nSamples;
   m_nAltCm = Pa2Cm(m_nPresPa,pzInx,tcInx);
   m_nAltCmx16 = m_nAltCm<<4;
   m_nAltM = (m_nAltCm+50)/100;
   m_nCmPerSecx16 = 0;
   m_nCmPerSec = 0;
   m_nSmpCnt = 0;
#ifdef USE_P_REGRESSION
   m_fTempC = float(m_nTempCx10)/10.0f;
#endif
   }

void CSensor::CalibrateT0()
   {
   int nT0;
   // Do the altitude computation using nominal values
   AcquireAveragedSample(4,SENSOR_DFL_PZTBL_SEL,SENSOR_DFL_TEMP_CORR_INX);
   nT0 = m_nTempCx10/10 + (65*m_nAltM)/10000;
   // Temp Corr LUT has range -10 to 40
   CLAMP(nT0,SENSOR_T0_MIN,SENSOR_T0_MAX);
   m_nTempCorrInx = nT0 + 10; 
   // now redo the altitude computation using the actual values
   AcquireAveragedSample(4,m_nPZTblSel,m_nTempCorrInx);
   }

void CSensor::GetAveragedAltitude(int nSamples)
   {
   AcquireAveragedSample(nSamples,m_nPZTblSel,m_nTempCorrInx);
   }

void CSensor::AveragePressureTemperature()
   {
   int n,pAccum,tAccum;
   tAccum = pAccum = 0;
   for (n = 0; n < m_nWindowSamples; n++)
      {
      pAccum += m_SmpBuf[n].p;
      tAccum += m_SmpBuf[n].t;
      }
   m_nPresPa = (pAccum+m_nWindowSamples/2)/m_nWindowSamples;
   m_nTempCx10 = (tAccum+m_nWindowSamples/2)/m_nWindowSamples;
#ifdef USE_P_REGRESSION
   m_fTempC = float(tAccum)/float(10.0f*m_nWindowSamples);
#endif
   }

#ifdef USE_P_REGRESSION
int CSensor::DeltaPressurePerSec()
    {
    int n;
    s64 dPa,sum_pt,sum_p,sum_t,sum_t2,trel,num,den,tref,p;

    // time reference is the oldest sample in window
    tref = s64(m_SmpBuf[m_nSmpCnt].tck);

    sum_pt = sum_p = sum_t = sum_t2 = 0;
    for (n = 0; n < m_nWindowSamples; n++)
        {
        p = m_SmpBuf[n].p;
        trel = (m_SmpBuf[n].tck - tref);
        sum_p += p;
        sum_t += trel;
        sum_t2 += (trel*trel);
        sum_pt += (trel*p);
        }

    num = (m_nWindowSamples*sum_pt) - (sum_p*sum_t);
    den = (m_nWindowSamples*sum_t2) - (sum_t*sum_t);

    dPa = den ? (num*TMR_TICKS_PER_SEC + den/2)/den : 0;
    return int(dPa);
    }
#endif

#ifdef USE_Z_REGRESSION
int CSensor::DeltaCmPerSec()
    {
    int n;
    s64 dZ,sum_zt,sum_z,sum_t,sum_t2,trel,num,den,tref,z;

    // time reference is the oldest sample in window
    tref = s64(m_SmpBuf[m_nSmpCnt].tck);

    sum_zt = sum_z = sum_t = sum_t2 = 0;
    for (n = 0; n < m_nWindowSamples; n++)
        {
        z = m_SmpBuf[n].z;
        trel = (m_SmpBuf[n].tck - tref);
        sum_z += z;
        sum_t += trel;
        sum_t2 += (trel*trel);
        sum_zt += (trel*z);
        }

    num = (m_nWindowSamples*sum_zt) - (sum_z*sum_t);
    den = (m_nWindowSamples*sum_t2) - (sum_t*sum_t);

    dZ = den ? (num*TMR_TICKS_PER_SEC + den/2)/den : 0;
    return int(dZ);
    }
#endif



void CSensor::ReadSampleContinuous()
   {
   int altCm,cmPerSec;
#ifdef USE_P_REGRESSION
   int dPa;
#endif

   switch (m_State)
      {
      case SENSOR_ST_IDLE :
      default :
      TriggerPressureSample();
      m_State = SENSOR_ST_PCONV;
      m_waitTick = WAIT_TICK(SENSOR_FOSP01_SAMPLE_DELAY_MS);
      ham_DrawText(0,0,"0 ");
      break;

      case SENSOR_ST_PCONV :
      ham_DrawText(1,0,"1");
      if (gTmr.ticks >= m_waitTick)
         {
         m_D1 = ReadPressure();
         TriggerTemperatureSample();
         m_State = SENSOR_ST_TCONV;
         m_waitTick = WAIT_TICK(SENSOR_FOSP01_SAMPLE_DELAY_MS);
         }
      break;

      case SENSOR_ST_TCONV :
      ham_DrawText(1,0,"2");
      if (gTmr.ticks >= m_waitTick)
         {
         m_D2 = ReadTemperature();
         // trigger the next sample immediately to maximize samples/sec
         TriggerPressureSample();
         m_waitTick = WAIT_TICK(SENSOR_FOSP01_SAMPLE_DELAY_MS);
         ConvertPressureTemperature(m_nPZTblSel,m_nTempCorrInx);
         AveragePressureTemperature();
         altCm = Pa2Cm(m_nPresPa,m_nPZTblSel,m_nTempCorrInx);
         LPFAltitude(altCm);
#ifdef USE_P_REGRESSION
         dPa = DeltaPressurePerSec();
         cmPerSec = ClimbRateCmPerSec(dPa,m_nPresPa);
#endif
#ifdef USE_Z_REGRESSION
         cmPerSec = DeltaCmPerSec();
#endif
         LPFClimbRate(cmPerSec);
         m_State = SENSOR_ST_PCONV;
         }
      break;
      }
   }


/*
 * $Header$
 */

