#include "i2c.h"
#include "utils.h"

extern CI2C gI2C;
extern CI2C gI2CB;

void CSensor::Init()
   {
#ifndef USE_SPLASH_SCREEN
   BusyWait(150); // wait at least 100mS for power to stabilize as per datasheet
#endif

   m_nSmpCnt1 = 0;
   m_nSmpCnt2 = 0;

   m_nKLPFA = 3;
   m_nKLPFC = SENSOR_DFL_KLPFC;
   m_nPZTblSel = SENSOR_DFL_PZTBL_SEL;
   m_nAltM = m_nAltCm = m_nAltCmx16 =  0;
   m_nCmPerSec = m_nCmPerSecx16 = 0;
   m_nWindowSecs = SENSOR_DFL_WINDOW_SECS;
   m_nWindowSamples = m_nWindowSecs * SENSOR_SAMPLES_PER_SEC;
   m_nTempCorrInx = SENSOR_DFL_TEMP_CORR_INX;

   // check startup procedure has completed
   if (gI2C.RcvByte(I2C_ID_SCP1000,0x07)&1)
      {
      ham_DrawText(0,0,"Err : SCP1 startup fail");
      while(1);
      }
   if (gI2CB.RcvByte(I2C_ID_SCP1000,0x07)&1)
      {
      ham_DrawText(0,0,"Err : SCP2 startup fail");
      while(1);
      }
      
   // check eeprom checksum
   if (gI2C.RcvByte(I2C_ID_SCP1000,0x7F) != 0x1)
      {
      ham_DrawText(0,0,"Err : SCP1 eep chksum fail");
      while(1);
      }
   if (gI2CB.RcvByte(I2C_ID_SCP1000,0x7F) != 0x1)
      {
      ham_DrawText(0,0,"Err : SCP2 eep chksum fail");
      while(1);
      }

   // low noise configuration
   gI2C.XmtByte(I2C_ID_SCP1000,0x02,0x2D);
   gI2C.XmtByte(I2C_ID_SCP1000,0x01,0x03);
   gI2C.XmtByte(I2C_ID_SCP1000,0x03,0x02);

   gI2CB.XmtByte(I2C_ID_SCP1000,0x02,0x2D);
   gI2CB.XmtByte(I2C_ID_SCP1000,0x01,0x03);
   gI2CB.XmtByte(I2C_ID_SCP1000,0x03,0x02);

   BusyWait(120); // wait at least 100mS
   }


void CSensor::InitData()
   {
   InitWindowBuffers(I2C_PORT1);
   InitWindowBuffers(I2C_PORT2);

   // use only one sensor for altitude & temperature data, as there is an uncertain
   // absolute offset between the two sensors due to the time interlacing
   // and sensor-to-sensor differences and noise.
   // However, the vario calculation uses both sensors with independent sample buffers
   
   m_nAltCm = Pa2Cm(m_nPresAvg1,m_nPZTblSel,m_nTempCorrInx);
   m_nAltCmx16 = m_nAltCm << 4;
   m_nAltM = (m_nAltCm+50)/100;
   m_nCmPerSecx16 = 0;
   m_nCmPerSec = 0;
#ifdef USE_P_REGRESSION
   m_fTempC = float(m_nTempAvg1)/10.0f;
#endif
   m_nTempCx10 = m_nTempAvg1;
   }


void CSensor::InitWindowBuffers(int port)
   {
   int n,tck;
#ifdef USE_Z_REGRESSION
   int z;
#endif
   SetSampleMode(port, SENSOR_STANDBY);
   SetSampleMode(port, SENSOR_SAMPLE_ONCE);
   AcquireAveragedSample(port,3,m_nPZTblSel,m_nTempCorrInx);
   tck = int(gTmr.ticks);
   if (port == I2C_PORT1)
      {
      m_nSmpCnt1 =  m_nWindowSamples-1;
      m_nPresAvg1 = m_nPresPaSample;
      m_nTempAvg1 = m_nTempCx10Sample;
#ifdef USE_Z_REGRESSION
      z = Pa2Cm(m_nPresAvg1,m_nPZTblSel,m_nTempCorrInx);
#endif
      n = m_nWindowSamples;
      while(n--)
         {
         m_SmpBuf1[n].p = m_nPresPaSample;
#ifdef USE_Z_REGRESSION
         m_SmpBuf1[n].z = z;
#endif
         m_SmpBuf1[n].t = m_nTempCx10Sample;
         m_SmpBuf1[n].tck = tck - m_nWindowSamples + n;
         }
      }
   else
      {
      m_nSmpCnt2 = m_nWindowSamples-1;
      m_nPresAvg2 = m_nPresPaSample;
      m_nTempAvg2 = m_nTempCx10Sample;
#ifdef USE_Z_REGRESSION
      z = Pa2Cm(m_nPresAvg2,m_nPZTblSel,m_nTempCorrInx);
#endif
      n = m_nWindowSamples;
      while(n--)
         {
         m_SmpBuf2[n].p = m_nPresPaSample;
#ifdef USE_Z_REGRESSION
         m_SmpBuf2[n].z = z;
#endif
         m_SmpBuf2[n].t = m_nTempCx10Sample;
         m_SmpBuf2[n].tck = tck - m_nWindowSamples + n;
         }
      }
   SetSampleMode(port,SENSOR_STANDBY);
   }


void CSensor::SetSampleMode(int port, int mode)
   {
   u8 bstatus;
   // check status register, if DRDY or RTERR is set
   // clear the status by reading the pressure data
   if (port == I2C_PORT1)
      bstatus = 0x30 & gI2C.RcvByte(I2C_ID_SCP1000,0x07);
   else
      bstatus = 0x30 & gI2CB.RcvByte(I2C_ID_SCP1000,0x07);
   if (bstatus)
       ReadPressure(port);

   m_nSampleMode = mode;
   switch (mode)
      {
      case SENSOR_SAMPLE_CONTINUOUS:
      // 15 bit resolution, 9 samples/sec
      if (port == I2C_PORT1)
         gI2C.XmtByte(I2C_ID_SCP1000,0x03,0x09);
      else
         gI2CB.XmtByte(I2C_ID_SCP1000,0x03,0x09);
      break;
      
      case SENSOR_SAMPLE_ONCE :
      if (port == I2C_PORT1)
         {
         // load CFG address 0x00
         gI2C.XmtByte(I2C_ID_SCP1000,0x02,0x00);
         // load CFG data (0x0D => 15 bit resolution)
         gI2C.XmtByte(I2C_ID_SCP1000,0x01,0x0D);
         // write data to address
         gI2C.XmtByte(I2C_ID_SCP1000,0x03,0x02);
         }
      else
         {
         // load CFG address 0x00
         gI2CB.XmtByte(I2C_ID_SCP1000,0x02,0x00);
         // load CFG data (0x0D => 15 bit resolution)
         gI2CB.XmtByte(I2C_ID_SCP1000,0x01,0x0D);
         // write data to address
         gI2CB.XmtByte(I2C_ID_SCP1000,0x03,0x02);
         }
      BusyWait(60);
      break;
      
      case SENSOR_STANDBY :
      default:
      if (port == I2C_PORT1)
         gI2C.XmtByte(I2C_ID_SCP1000,0x03,0x00);
      else
         gI2CB.XmtByte(I2C_ID_SCP1000,0x03,0x00);
      BusyWait(60);
      break;
      }
   }


void CSensor::TriggerSample(int port)
   {
   if (port == I2C_PORT1)
      gI2C.XmtByte(I2C_ID_SCP1000,0x03,0x0C);
   else
      gI2CB.XmtByte(I2C_ID_SCP1000,0x03,0x0C);
   }


int CSensor::AcquireAveragedSample(int port, int nSamples, int pzInx, int tcInx)
   {
   u8 bstatus;
   u32 timeoutTick;
   int pAccum,tAccum,cnt;
   
   pAccum = 0;
   tAccum = 0;
   cnt = 0;
   while (nSamples--)
      {
      bstatus = 0;
      TriggerSample(port);
      BusyWait(135);
      timeoutTick = WAIT_TICK(100);
      do
         {
         BusyWait(10);
         if (port == I2C_PORT1)
            bstatus = 0x20 & gI2C.RcvByte(I2C_ID_SCP1000,0x07);
         else
            bstatus = 0x20 & gI2CB.RcvByte(I2C_ID_SCP1000,0x07);
         } while ((bstatus == 0) && (gTmr.ticks < timeoutTick));
      
      if (bstatus)
         {
         cnt++;
         pAccum += ReadPressure(port);
         tAccum += ReadTemperature(port);
         }
      BusyWait(10);
      }

   if (cnt)
      {
      m_nPresPaSample = (pAccum+cnt/2)/cnt;
      m_nTempCx10Sample = tAccum > 0 ? (tAccum+cnt/2)/cnt : (tAccum-cnt/2)/cnt;
      if (port == I2C_PORT1)
         {
         m_nAltCm = Pa2Cm(m_nPresPaSample,pzInx,tcInx);
         m_nAltCmx16 = m_nAltCm<<4;
         m_nAltM = (m_nAltCm+50)/100;
         m_nTempCx10 = m_nTempCx10Sample;
         }
      return 0;
      }
   else
      {
      m_nPresPaSample = 0;
      m_nTempCx10Sample = 0;
      if (port == I2C_PORT1)
         {
         m_nAltCm = 0;
         m_nAltCmx16 = 0;
         m_nAltM = 5555;
         m_nTempCx10 = 0;
         }
      return 1;
      }
   }

void CSensor::CalibrateT0()
   {
   int nT0;
   SetSampleMode(I2C_PORT1, SENSOR_STANDBY);
   SetSampleMode(I2C_PORT1, SENSOR_SAMPLE_ONCE);
   // Do the altitude computation using nominal values
   AcquireAveragedSample(I2C_PORT1,4,SENSOR_DFL_PZTBL_SEL,SENSOR_DFL_TEMP_CORR_INX);
   nT0 = m_nTempCx10/10 + (65*m_nAltM)/10000;
   // Temp Corr LUT has range -10 to 40
   CLAMP(nT0,SENSOR_T0_MIN,SENSOR_T0_MAX);
   m_nTempCorrInx = nT0 + 10; 
   // now redo the altitude computation using the actual values
   AcquireAveragedSample(I2C_PORT1,4,m_nPZTblSel,m_nTempCorrInx);
   }

void CSensor::GetAveragedAltitude(int nSamples)
   {
   AcquireAveragedSample(I2C_PORT1,nSamples,m_nPZTblSel,m_nTempCorrInx);
   }

// for timing the sample cycle, I found that SCP1 sample interval is about 119mS,
// SCP2 sample interval is about 112mS. Close to datasheet claim of 9 samples/second

//int sTick1;
//int sTick2;

void CSensor::ReadSampleContinuous()
   {
   u8 bstatus;
   int altCm,cmPerSec;
#ifdef USE_P_REGRESSION
   int deltaPressure;
#endif
   static u32 tick;
   
   // check only once every tick
   if (gTmr.ticks != tick)
      {
      tick = gTmr.ticks;
      bstatus = 0x20 & gI2C.RcvByte(I2C_ID_SCP1000,0x07);
      if (bstatus)
         {
         m_SmpBuf1[m_nSmpCnt1].tck = (int)gTmr.ticks;
         m_SmpBuf1[m_nSmpCnt1].t = ReadTemperature(I2C_PORT1);
         m_SmpBuf1[m_nSmpCnt1].p = ReadPressure(I2C_PORT1);
#ifdef USE_Z_REGRESSION
         m_SmpBuf1[m_nSmpCnt1].z = Pa2Cm(m_SmpBuf1[m_nSmpCnt1].p,m_nPZTblSel,m_nTempCorrInx);
#endif
         AveragePressureTemperature(I2C_PORT1);
         altCm = Pa2Cm(m_nPresAvg1,m_nPZTblSel,m_nTempCorrInx);
         LPFAltitude(altCm);

#ifdef USE_P_REGRESSION
         deltaPressure = DeltaPressurePerSec(I2C_PORT1);
         cmPerSec = ClimbRateCmPerSec(deltaPressure,m_nPresAvg1);
#endif
#ifdef USE_Z_REGRESSION
         cmPerSec = DeltaCmPerSec(I2C_PORT1);
#endif
         LPFClimbRate(cmPerSec);
         m_nSmpCnt1++;
         if (m_nSmpCnt1 == m_nWindowSamples)
            {
//            ham_DrawText(0,0,"t%06d",((gTmr.ticks-sTick1)*TMR_TICK_MS)/m_nWindowSamples);
            m_nSmpCnt1 = 0;
//            sTick1 = gTmr.ticks;
            }
         }

      bstatus = 0x20 & gI2CB.RcvByte(I2C_ID_SCP1000,0x07);
      if (bstatus)
         {
         m_SmpBuf2[m_nSmpCnt2].tck = (int)gTmr.ticks;
         m_SmpBuf2[m_nSmpCnt2].t = ReadTemperature(I2C_PORT2);
         m_SmpBuf2[m_nSmpCnt2].p = ReadPressure(I2C_PORT2);
#ifdef USE_Z_REGRESSION
         m_SmpBuf2[m_nSmpCnt2].z = Pa2Cm(m_SmpBuf2[m_nSmpCnt2].p,m_nPZTblSel,m_nTempCorrInx);
#endif
       /**** Note no calculation of altitude or temperature using sensor 2 ****/
         
#ifdef USE_P_REGRESSION
         deltaPressure = DeltaPressurePerSec(I2C_PORT2);
         cmPerSec = ClimbRateCmPerSec(deltaPressure,m_nPresAvg2);
#endif
#ifdef USE_Z_REGRESSION
         cmPerSec = DeltaCmPerSec(I2C_PORT2);
#endif
         LPFClimbRate(cmPerSec);
         m_nSmpCnt2++;
         if (m_nSmpCnt2 == m_nWindowSamples)
            {
           // ham_DrawText(0,1,"t%06d",((gTmr.ticks-sTick2)*TMR_TICK_MS)/m_nWindowSamples);
            m_nSmpCnt2 = 0;
           // sTick2 = gTmr.ticks;
            }
         }
      }
   }


int CSensor::ReadTemperature(int port)
{
   u16 word;
   u8  byte[2];
   int temp;
   
   if (port == I2C_PORT1)
      gI2C.RcvBuf(I2C_ID_SCP1000,0x81,2,byte);
   else
      gI2CB.RcvBuf(I2C_ID_SCP1000,0x81,2,byte);
   word = ((((u16)byte[0])<<8)| ((u16)byte[1])) & 0x3FFF;
   if (word & 0x2000)
      {
      word = (~word&0x3FFF) + 1;
      temp = -((int)word);
      }
   else
      {
      temp = ((int)word);
      }
   // scp1000 reading is celsius x 20
   // return rounded up celsius x 10 reading
   return (temp > 0 ? (temp+1)/2 : (temp-1)/2);
}



int CSensor::ReadPressure(int port)
{
   u8 byte[3];
   u32 dword;
   
   if (port == I2C_PORT1)
      {
      byte[0] = gI2C.RcvByte(I2C_ID_SCP1000,0x7F);
      gI2C.RcvBuf(I2C_ID_SCP1000,0x80,2,&byte[1]);
      }
   else
      {
      byte[0] = gI2CB.RcvByte(I2C_ID_SCP1000,0x7F);
      gI2CB.RcvBuf(I2C_ID_SCP1000,0x80,2,&byte[1]);
      }
   dword = (((u32)(byte[0]&0x7))<<16) | (((u32)byte[1])<<8) | (u32)byte[2];
   // 19 bit result, only 15 MSbits are significant
   // return result in Pascals
   dword = (dword & u32(0x7FFF0))>>2;
   return int(dword);
}



void CSensor::AveragePressureTemperature(int port)
   {
   int n, pAccum, tAccum;

   tAccum = pAccum = 0;
   if (port == I2C_PORT1)
      {
      for (n = 0; n < m_nWindowSamples; n++)
         {
         tAccum += m_SmpBuf1[n].t;
         pAccum += m_SmpBuf1[n].p;
         }
      m_nPresAvg1 = (pAccum+m_nWindowSamples/2)/m_nWindowSamples;
      m_nTempAvg1 = (tAccum > 0 ? (tAccum+m_nWindowSamples/2)/m_nWindowSamples : (tAccum-m_nWindowSamples/2)/m_nWindowSamples);
      m_nTempCx10 = m_nTempAvg1;
#ifdef USE_P_REGRESSION
      m_fTempC = float(m_nTempAvg1)/10.0f;
#endif
      }
   else
      {
      for (n = 0; n < m_nWindowSamples; n++)
         {
         tAccum += m_SmpBuf2[n].t;
         pAccum += m_SmpBuf2[n].p;
         }
      m_nPresAvg2 = (pAccum+m_nWindowSamples/2)/m_nWindowSamples;
      m_nTempAvg2 = (tAccum > 0 ? (tAccum+m_nWindowSamples/2)/m_nWindowSamples : (tAccum-m_nWindowSamples/2)/m_nWindowSamples);
      }
      
   return;
   }


/*
This pressure->altitude calculation (instead of the LUT approach)
takes approx 24mS to execute on Gameboy Advance!!
int Pa2CmExponent(int pa)
   {
   float fpratio;
   float fcm;
   fpratio = float(pa)/101325.0f;
   fcm = 4433076.9*(1.0 - pow(double(fpratio),0.190275));
   return fcm > 0.0f ? int(fcm+0.5f) : int(fcm-0.5f);
   }
*/

#ifdef USE_P_REGRESSION
int CSensor::DeltaPressurePerSec(int port)
    {
    int n;
    s64 sum_pt,sum_p,sum_t,sum_t2,trel,num,den,p,dP,tref;

    tref = s64(port == I2C_PORT1 ? m_SmpBuf1[m_nSmpCnt1].tck : m_SmpBuf2[m_nSmpCnt2].tck);

    // do a linear regression to calculate dP/tck
    sum_pt = sum_p = sum_t = sum_t2 = 0;
    if (port == I2C_PORT1)
       {
       for (n = 0; n < m_nWindowSamples; n++)
          {
          p = m_SmpBuf1[n].p;
          trel = (m_SmpBuf1[n].tck - tref);
          sum_p += p;
          sum_t += trel;
          sum_t2 += (trel*trel);
          sum_pt += (trel*p);
          }
       }
    else
       {
       for (n = 0; n < m_nWindowSamples; n++)
          {
          p = m_SmpBuf2[n].p;
          trel = (m_SmpBuf2[n].tck - tref);
          sum_p += p;
          sum_t += trel;
          sum_t2 += (trel*trel);
          sum_pt += (trel*p);
          }
       }

    num = (m_nWindowSamples*sum_pt) - (sum_p*sum_t);
    den = (m_nWindowSamples*sum_t2)- (sum_t*sum_t);

    dP = den ? (num*TMR_TICKS_PER_SEC +den/2)/den : 0;
    return int(dP);
    }
#endif



#ifdef USE_Z_REGRESSION

int CSensor::DeltaCmPerSec(int port)
    {
    int n;
    s64 sum_zt,sum_z,sum_t,sum_t2,trel,num,den,z,dZ,tref;

    tref = s64(port == I2C_PORT1 ? m_SmpBuf1[m_nSmpCnt1].tck : m_SmpBuf2[m_nSmpCnt2].tck);

    // do a linear regression to calculate dZ/tck
    sum_zt = sum_z = sum_t = sum_t2 = 0;
    if (port == I2C_PORT1)
       {
       for (n = 0; n < m_nWindowSamples; n++)
          {
          z = m_SmpBuf1[n].z;
          trel = (m_SmpBuf1[n].tck - tref);
          sum_z += z;
          sum_t += trel;
          sum_t2 += (trel*trel);
          sum_zt += (trel*z);
          }
       }
    else
       {
       for (n = 0; n < m_nWindowSamples; n++)
          {
          z = m_SmpBuf2[n].z;
          trel = (m_SmpBuf2[n].tck - tref);
          sum_z += z;
          sum_t += trel;
          sum_t2 += (trel*trel);
          sum_zt += (trel*z);
          }
       }

    num = (m_nWindowSamples*sum_zt) - (sum_z*sum_t);
    den = (m_nWindowSamples*sum_t2)- (sum_t*sum_t);

    dZ = (den ? (num*TMR_TICKS_PER_SEC)/den : 0);
    return int(dZ);
    }
#endif

/*
 * $Header$
 */



