#include "def.h"
#include "sensor.h"
#include "tmr.h"

#ifdef SENSOR_FOSP01
#include "fosp01.h"
#include "fosp01.cpp"
#endif

#ifdef SENSOR_MS5534
#include "ms5534.h"
#include "ms5534.cpp"
#endif

#ifdef SENSOR_SCP1000
#include "scp1000.h"
#include "scp1000.cpp"
#endif

// 1976 US Standard Atmosphere model for converting pressure to altitude
// uses a nominal mean sea level temperature of 15 degrees Celsius. 
// This table contains a multiplicative correction factor n/1024 for
// mean sea level temperatures in the range -10 to 40 degrees Celsius.

const int gnTempCorrLUT[51] = {
935, // 0 : -10C
939,
942,
946,
949,
953, // 5 : -5C
956,
960,
964,
967,
971, // 10 : 0C
974,
978,
981,
985,
988, // 15 : 5C
992,
996,
999,
1003,
1006, // 20 : 10C
1010,
1013,
1017,
1020,
1024, // 25 : 15C
1028,
1031,
1035,
1038,
1042, // 30 : 20C
1045,
1049,
1052,
1056,
1060, // 35 : 25C
1063,
1067,
1070,
1074,
1077, // 40 : 30C
1081,
1084,
1088,
1092,
1095, // 45 : 35C
1099,
1102,
1106,
1109,
1113  // 50 : 40C
};


#ifdef USE_P_REGRESSION
// only this function and the temperature average calculation uses
// floating point arithmetic

int CSensor::ClimbRateCmPerSec(int dPa, int Pa)
   {
   float fcmPerSec;
   float fKelvin;
   int   cmPerSec;

   fKelvin = m_fTempC + 273.15f;
   fcmPerSec = -(2927.16f*fKelvin*float(dPa))/float(Pa);
   cmPerSec = fcmPerSec > 0.0f ? int(fcmPerSec+0.5f) : int(fcmPerSec-0.5f);
   return cmPerSec;
   }
#endif

void CSensor::LPFClimbRate(int cmPerSec)
   {
   int tmp;
   tmp = m_nCmPerSecx16*(16-m_nKLPFC) + (cmPerSec<<4)*m_nKLPFC;
   m_nCmPerSecx16 = tmp >= 0 ? ((tmp + 8)>>4) : ((tmp - 8)>>4);
   m_nCmPerSec = m_nCmPerSecx16>>4;
   CORE(m_nCmPerSec,VARIO_CORE_CLIMB_RATE_CM);
   CLAMP(m_nCmPerSec,-VARIO_MAX_CLIMB_RATE_LOG_CM,VARIO_MAX_CLIMB_RATE_LOG_CM);
   }

void CSensor::LPFAltitude(int altCm)
   {
   m_nAltCmx16 = (m_nAltCmx16*(16-m_nKLPFA) + (altCm<<4)*m_nKLPFA + 8)>>4;
   m_nAltCm = m_nAltCmx16>>4;

   m_nAltM = (m_nAltCm + 50)/100;
   CLAMP(m_nAltM,SENSOR_MIN_ALTITUDE_M,SENSOR_MAX_ALTITUDE_M);
   }

// Set of 40 pressure-altitude LUTs, each computed for a different mean sea level 
// reference pressure, varying from 100325Pa (LUT0) to 102275Pa (LUT39), with a 
// step of 50Pa. LUT20 has the nominal mean sea level reference 
// pressure of 101325Pa specified by the 1976 US Standard Atmosphere model.
// These tables have been computed for a standard mean sea level temperature of 
// 15 degrees Celsius. The temperature correction LUT gnTempCorrLUT[] contains
// the correction factor for other temperatures in the range -10 to 40 degrees Celsius.


#include "pztbl15.txt"


int CSensor::Pa2Cm( int pa, int pzInx, int tcInx )
   {
   int inx,pa1,pa2,z1,z2,z;

   inx = 0;
   pa2 = PA_INIT;

   // table of decreasing pressures, find the nearest
   // entries p1, p2 where p1 > pa > p2
    
   while (inx < PZLUT_ENTRIES && pa < pa2)
      {
      inx++;
      pa2 -= PA_DELTA;
      }

   if (inx == PZLUT_ENTRIES)
      {
      z = 999900;
      }
   else
      {
      pa1 = (inx > 0 ? pa2+PA_DELTA : pa2);
      z1 = (inx > 0 ? gPZTbl[pzInx][inx-1] : gPZTbl[pzInx][0]);
      z2 = gPZTbl[pzInx][inx];
      z = ((pa1-pa)*z2 + (pa-pa2)*z1)>>10;
      }

   // Correct for mean sea level temperature
   z = (z*gnTempCorrLUT[tcInx])>>10;
   return z;
   }


/*
 * $Header: $
 */

